@@ -444,7 +444,7 @@ static int set_network_active(struct connman_network *network,
 
 	error = set_property(path, OFONO_CONTEXT_INTERFACE,
 				"Active", DBUS_TYPE_BOOLEAN, &active,
-				set_active_reply, g_strdup(path), NULL);
+				set_active_reply, g_strdup(path), g_free);
 
 	if (active == FALSE && error == -EINPROGRESS)
 		error = 0;
@@ -452,25 +452,6 @@ static int set_network_active(struct connman_network *network,
 	return error;
 }
 
-static void set_apn(struct connman_network *network)
-{
-	const char *apn, *path;
-
-	apn = connman_network_get_string(network, "Cellular.APN");
-	if (apn == NULL)
-		return;
-
-	path = connman_network_get_string(network, "Path");
-	if (path == NULL)
-		return;
-
-	DBG("path %s, apn %s", path, apn);
-
-	set_property(path, OFONO_CONTEXT_INTERFACE,
-			"AccessPointName", DBUS_TYPE_STRING, &apn,
-			NULL, NULL, NULL);
-}
-
 static int network_connect(struct connman_network *network)
 {
 	struct connman_device *device;
@@ -508,6 +489,8 @@ static int network_disconnect(struct connman_network *network)
 	if (connman_network_get_index(network) < 0)
 		return -ENOTCONN;
 
+	connman_network_set_associating(network, FALSE);
+
 	return set_network_active(network, FALSE);
 }
 
@@ -520,14 +503,6 @@ static void network_remove(struct connman_network *network)
 	g_hash_table_remove(network_hash, path);
 }
 
-static int network_setup(struct connman_network *network, const char *key)
-{
-	if (g_strcmp0(key, "Cellular.APN") == 0)
-		set_apn(network);
-
-	return 0;
-}
-
 static struct connman_network_driver network_driver = {
 	.name		= "network",
 	.type		= CONNMAN_NETWORK_TYPE_CELLULAR,
@@ -535,7 +510,6 @@ static struct connman_network_driver network_driver = {
 	.remove		= network_remove,
 	.connect	= network_connect,
 	.disconnect	= network_disconnect,
-	.setup		= network_setup,
 };
 
 static void update_settings(DBusMessageIter *array,
@@ -615,12 +589,6 @@ static int add_network(struct connman_device *device,
 				DBG("path %p type %s", path, type);
 				goto error;
 			}
-		} else if (g_str_equal(key, "AccessPointName")) {
-			const char *ap;
-
-			dbus_message_iter_get_basic(&value, &ap);
-
-			connman_network_set_string(network, "Cellular.APN", ap);
 		} else if (g_str_equal(key, "Settings"))
 			update_settings(&value, network);
 		else if (g_str_equal(key, "Active") == TRUE)
@@ -1456,6 +1424,63 @@ static gboolean gprs_changed(DBusConnection *connection, DBusMessage *message,
 	return TRUE;
 }
 
+static gboolean context_added(DBusConnection *connection,
+				DBusMessage *message, void *user_data)
+{
+	const char *path = dbus_message_get_path(message);
+	const char *network_path;
+	struct modem_data *modem;
+	DBusMessageIter iter, properties;
+
+	DBG("path %s", path);
+
+	modem = g_hash_table_lookup(modem_hash, path);
+	if (modem == NULL || modem->device == NULL)
+		return TRUE;
+
+	if (dbus_message_iter_init(message, &iter) == FALSE)
+		return TRUE;
+
+	dbus_message_iter_get_basic(&iter, &network_path);
+
+	dbus_message_iter_next(&iter);
+	dbus_message_iter_recurse(&iter, &properties);
+
+	add_network(modem->device, network_path, &properties);
+
+	return TRUE;
+}
+
+static gboolean context_removed(DBusConnection *connection,
+				DBusMessage *message, void *user_data)
+{
+	const char *path = dbus_message_get_path(message);
+	const char *network_path, *identifier;
+	struct modem_data *modem;
+	struct connman_network *network;
+	DBusMessageIter iter;
+
+	DBG("path %s", path);
+
+	modem = g_hash_table_lookup(modem_hash, path);
+	if (modem == NULL || modem->device == NULL)
+		return TRUE;
+
+	if (dbus_message_iter_init(message, &iter) == FALSE)
+		return TRUE;
+
+	dbus_message_iter_get_basic(&iter, &network_path);
+
+	network = g_hash_table_lookup(network_hash, network_path);
+	if (network == NULL)
+		return TRUE;
+
+	identifier = connman_network_get_identifier(network);
+	connman_device_remove_network(modem->device, identifier);
+
+	return TRUE;
+}
+
 static gboolean modem_added(DBusConnection *connection,
 				DBusMessage *message, void *user_data)
 {
@@ -1716,6 +1741,8 @@ static guint watch;
 static guint reg_watch;
 static guint sim_watch;
 static guint gprs_watch;
+static guint context_added_watch;
+static guint context_removed_watch;
 static guint modem_watch;
 static guint modem_added_watch;
 static guint modem_removed_watch;
@@ -1744,6 +1771,18 @@ static int ofono_init(void)
 						gprs_changed,
 						NULL, NULL);
 
+	context_added_watch = g_dbus_add_signal_watch(connection, NULL, NULL,
+						OFONO_GPRS_INTERFACE,
+						CONTEXT_ADDED,
+						context_added,
+						NULL, NULL);
+
+	context_removed_watch = g_dbus_add_signal_watch(connection, NULL, NULL,
+						OFONO_GPRS_INTERFACE,
+						CONTEXT_REMOVED,
+						context_removed,
+						NULL, NULL);
+
 	modem_watch = g_dbus_add_signal_watch(connection, NULL, NULL,
 						OFONO_MODEM_INTERFACE,
 						PROPERTY_CHANGED,
@@ -1774,7 +1813,8 @@ static int ofono_init(void)
 						context_changed,
 						NULL, NULL);
 
-	if (watch == 0 || gprs_watch == 0 || modem_watch == 0 ||
+	if (watch == 0 || gprs_watch == 0 || context_added_watch == 0 ||
+			context_removed_watch == 0 || modem_watch == 0 ||
 			reg_watch == 0 || sim_watch == 0 ||
 			modem_added_watch == 0 || modem_removed_watch == 0 ||
 				context_watch == 0) {
@@ -1799,6 +1839,8 @@ remove:
 	g_dbus_remove_watch(connection, sim_watch);
 	g_dbus_remove_watch(connection, reg_watch);
 	g_dbus_remove_watch(connection, gprs_watch);
+	g_dbus_remove_watch(connection, context_added_watch);
+	g_dbus_remove_watch(connection, context_removed_watch);
 	g_dbus_remove_watch(connection, modem_watch);
 	g_dbus_remove_watch(connection, modem_added_watch);
 	g_dbus_remove_watch(connection, modem_removed_watch);
@@ -1815,6 +1857,8 @@ static void ofono_exit(void)
 	g_dbus_remove_watch(connection, sim_watch);
 	g_dbus_remove_watch(connection, reg_watch);
 	g_dbus_remove_watch(connection, gprs_watch);
+	g_dbus_remove_watch(connection, context_added_watch);
+	g_dbus_remove_watch(connection, context_removed_watch);
 	g_dbus_remove_watch(connection, modem_watch);
 	g_dbus_remove_watch(connection, modem_added_watch);
 	g_dbus_remove_watch(connection, modem_removed_watch);
