@@ -218,11 +218,24 @@ void __connman_ipconfig_delroute(int index, int family, unsigned char scope,
 
 void __connman_ipconfig_foreach(void (*function) (int index, void *user_data),
 							void *user_data);
-unsigned short __connman_ipconfig_get_type(int index);
-unsigned int __connman_ipconfig_get_flags(int index);
-const char *__connman_ipconfig_get_gateway(int index);
+enum connman_ipconfig_type __connman_ipconfig_get_config_type(
+					struct connman_ipconfig *ipconfig);
+unsigned short __connman_ipconfig_get_type_from_index(int index);
+unsigned int __connman_ipconfig_get_flags_from_index(int index);
+const char *__connman_ipconfig_get_gateway_from_index(int index);
 void __connman_ipconfig_set_index(struct connman_ipconfig *ipconfig, int index);
 
+const char *__connman_ipconfig_get_local(struct connman_ipconfig *ipconfig);
+void __connman_ipconfig_set_local(struct connman_ipconfig *ipconfig, const char *address);
+const char *__connman_ipconfig_get_peer(struct connman_ipconfig *ipconfig);
+void __connman_ipconfig_set_peer(struct connman_ipconfig *ipconfig, const char *address);
+const char *__connman_ipconfig_get_broadcast(struct connman_ipconfig *ipconfig);
+void __connman_ipconfig_set_broadcast(struct connman_ipconfig *ipconfig, const char *broadcast);
+const char *__connman_ipconfig_get_gateway(struct connman_ipconfig *ipconfig);
+void __connman_ipconfig_set_gateway(struct connman_ipconfig *ipconfig, const char *gateway);
+unsigned char __connman_ipconfig_get_prefixlen(struct connman_ipconfig *ipconfig);
+void __connman_ipconfig_set_prefixlen(struct connman_ipconfig *ipconfig, unsigned char prefixlen);
+
 int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig);
 int __connman_ipconfig_disable(struct connman_ipconfig *ipconfig);
 
@@ -248,8 +261,6 @@ void __connman_ipconfig_set_element_ipv6_gateway(
 			struct connman_ipconfig *ipconfig,
 				struct connman_element *element);
 
-int __connman_ipconfig_set_gateway(struct connman_ipconfig *ipconfig,
-					struct connman_element *parent);
 int __connman_ipconfig_set_address(struct connman_ipconfig *ipconfig);
 int __connman_ipconfig_clear_address(struct connman_ipconfig *ipconfig);
 unsigned char __connman_ipconfig_netmask_prefix_len(const char *netmask);
@@ -282,6 +293,12 @@ void __connman_ipv4_cleanup(void);
 int __connman_connection_init(void);
 void __connman_connection_cleanup(void);
 
+int __connman_connection_gateway_add(struct connman_service *service,
+					const char *ipv4_gateway,
+					const char *ipv6_gateway,
+					const char *peer);
+void __connman_connection_gateway_remove(struct connman_service *service);
+
 gboolean __connman_connection_update_gateway(void);
 
 int __connman_wpad_init(void);
@@ -298,8 +315,8 @@ void __connman_technology_list(DBusMessageIter *iter, void *user_data);
 
 int __connman_technology_add_device(struct connman_device *device);
 int __connman_technology_remove_device(struct connman_device *device);
-int __connman_technology_enable_device(struct connman_device *device);
-int __connman_technology_disable_device(struct connman_device *device);
+int __connman_technology_enable(enum connman_service_type type);
+int __connman_technology_disable(enum connman_service_type type);
 int __connman_technology_add_rfkill(unsigned int index,
 					enum connman_service_type type,
 						connman_bool_t softblock,
@@ -476,7 +493,8 @@ int __connman_service_set_immutable(struct connman_service *service,
 void __connman_service_set_string(struct connman_service *service,
 					const char *key, const char *value);
 int __connman_service_indicate_state(struct connman_service *service,
-					enum connman_service_state state);
+					enum connman_service_state new_state,
+					enum connman_ipconfig_type type);
 int __connman_service_indicate_error(struct connman_service *service,
 					enum connman_service_error error);
 int __connman_service_clear_error(struct connman_service *service);
@@ -492,10 +510,10 @@ struct connman_service *__connman_service_connect_type(enum connman_service_type
 
 const char *__connman_service_type2string(enum connman_service_type type);
 
-void __connman_service_append_nameserver(struct connman_service *service,
-						const char *nameserver);
-void __connman_service_remove_nameserver(struct connman_service *service,
-						const char *nameserver);
+int __connman_service_nameserver_append(struct connman_service *service,
+					const char *nameserver);
+int __connman_service_nameserver_remove(struct connman_service *service,
+					const char *nameserver);
 void __connman_service_nameserver_add_routes(struct connman_service *service,
 						const char *gw);
 void __connman_service_nameserver_del_routes(struct connman_service *service);
