@@ -66,8 +66,6 @@ static const char *type2string(enum connman_element_type type)
 		return "bootp";
 	case CONNMAN_ELEMENT_TYPE_ZEROCONF:
 		return "zeroconf";
-	case CONNMAN_ELEMENT_TYPE_CONNECTION:
-		return "connection";
 	case CONNMAN_ELEMENT_TYPE_VENDOR:
 		return "vendor";
 	}
@@ -196,8 +194,11 @@ struct connman_service *__connman_element_get_service(struct connman_element *el
 	enum connman_device_type type;
 
 	device = __connman_element_get_device(element);
-	if (device == NULL)
-		return NULL;
+	if (device == NULL) {
+		/* Workaround for the connection removal. */
+		service = __connman_service_lookup_from_index(element->index);
+		return service;
+	}
 
 	type = connman_device_get_type(device);
 
@@ -205,6 +206,7 @@ struct connman_service *__connman_element_get_service(struct connman_element *el
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
 	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_GADGET:
 		break;
 	case CONNMAN_DEVICE_TYPE_ETHERNET:
 	case CONNMAN_DEVICE_TYPE_WIFI:
@@ -233,29 +235,6 @@ struct connman_device *__connman_element_get_device(struct connman_element *elem
 	return __connman_element_get_device(element->parent);
 }
 
-const char *__connman_element_get_device_path(struct connman_element *element)
-{
-	struct connman_device *device;
-
-	device = __connman_element_get_device(element);
-	if (device == NULL)
-		return NULL;
-
-	return connman_device_get_path(device);
-}
-
-const char *__connman_element_get_network_path(struct connman_element *element)
-{
-	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK &&
-						element->network != NULL)
-		return element->path;
-
-	if (element->parent == NULL)
-		return NULL;
-
-	return __connman_element_get_network_path(element->parent);
-}
-
 struct find_data {
 	enum connman_service_type type;
 	struct connman_device *device;
@@ -313,6 +292,7 @@ static gboolean request_scan(GNode *node, gpointer user_data)
 	case CONNMAN_SERVICE_TYPE_CELLULAR:
 	case CONNMAN_SERVICE_TYPE_GPS:
 	case CONNMAN_SERVICE_TYPE_VPN:
+	case CONNMAN_SERVICE_TYPE_GADGET:
 		return FALSE;
 	case CONNMAN_SERVICE_TYPE_WIFI:
 	case CONNMAN_SERVICE_TYPE_WIMAX:
@@ -357,6 +337,7 @@ static gboolean enable_technology(GNode *node, gpointer user_data)
 	case CONNMAN_SERVICE_TYPE_SYSTEM:
 	case CONNMAN_SERVICE_TYPE_GPS:
 	case CONNMAN_SERVICE_TYPE_VPN:
+	case CONNMAN_SERVICE_TYPE_GADGET:
 		return FALSE;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
@@ -409,6 +390,7 @@ static gboolean disable_technology(GNode *node, gpointer user_data)
 	case CONNMAN_SERVICE_TYPE_SYSTEM:
 	case CONNMAN_SERVICE_TYPE_GPS:
 	case CONNMAN_SERVICE_TYPE_VPN:
+	case CONNMAN_SERVICE_TYPE_GADGET:
 		return FALSE;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
@@ -616,6 +598,8 @@ struct connman_element *connman_element_create(const char *name)
 
 	__connman_element_initialize(element);
 
+	element->name = g_strdup(name);
+
 	return element;
 }
 
@@ -777,6 +761,12 @@ int connman_element_get_value(struct connman_element *element,
 								id, value);
 		*((char **) value) = element->ipv4.address;
 		break;
+	case CONNMAN_PROPERTY_ID_IPV4_PEER:
+		if (element->ipv4.peer == NULL)
+			return connman_element_get_value(element->parent,
+								id, value);
+		*((char **) value) = element->ipv4.peer;
+		break;
 	case CONNMAN_PROPERTY_ID_IPV4_NETMASK:
 		if (element->ipv4.netmask == NULL)
 			return connman_element_get_value(element->parent,
@@ -1322,6 +1312,28 @@ void connman_element_set_error(struct connman_element *element,
 	__connman_service_indicate_error(service, convert_error(error));
 }
 
+void __connman_element_set_driver(struct connman_element *element)
+{
+	GSList *list;
+
+	DBG("element %p name %s driver %p", element, element->name,
+						element->driver);
+
+	if (element->driver)
+		return;
+
+	for (list = driver_list; list; list = list->next) {
+		struct connman_driver *driver = list->data;
+
+		if (match_driver(element, driver) == FALSE)
+			continue;
+
+		element->driver = driver;
+
+		break;
+	}
+}
+
 int __connman_element_init(const char *device, const char *nodevice)
 {
 	struct connman_element *element;
@@ -1385,12 +1397,11 @@ void __connman_element_start(void)
 	started = TRUE;
 
 	__connman_rtnl_start();
-	__connman_udev_start();
 
-	__connman_connection_init();
 	__connman_ipv4_init();
 	__connman_dhcp_init();
 	__connman_wpad_init();
+	__connman_wispr_init();
 
 	__connman_rfkill_init();
 }
@@ -1401,11 +1412,11 @@ void __connman_element_stop(void)
 
 	__connman_rfkill_cleanup();
 
+	__connman_wispr_cleanup();
 	__connman_wpad_cleanup();
 	__connman_dhcp_cleanup();
 	__connman_ipv4_cleanup();
 	__connman_provider_cleanup();
-	__connman_connection_cleanup();
 }
 
 static gboolean free_driver(GNode *node, gpointer data)
@@ -1453,6 +1464,8 @@ void __connman_element_cleanup(void)
 	g_node_traverse(element_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
 							free_node, NULL);
 
+	connman_element_unref(element_root->data);
+
 	g_node_destroy(element_root);
 	element_root = NULL;
 
