@@ -243,7 +243,7 @@ unsigned int connman_rtnl_add_newlink_watch(int index,
 	DBG("id %d", watch->id);
 
 	if (callback) {
-		unsigned int flags = __connman_ipconfig_get_flags(index);
+		unsigned int flags = __connman_ipconfig_get_flags_from_index(index);
 
 		if (flags > 0)
 			callback(flags, 0, user_data);
@@ -283,14 +283,14 @@ static void trigger_rtnl(int index, void *user_data)
 	struct connman_rtnl *rtnl = user_data;
 
 	if (rtnl->newlink) {
-		unsigned short type = __connman_ipconfig_get_type(index);
-		unsigned int flags = __connman_ipconfig_get_flags(index);
+		unsigned short type = __connman_ipconfig_get_type_from_index(index);
+		unsigned int flags = __connman_ipconfig_get_flags_from_index(index);
 
 		rtnl->newlink(type, index, flags, 0);
 	}
 
 	if (rtnl->newgateway) {
-		const char *gateway = __connman_ipconfig_get_gateway(index);
+		const char *gateway = __connman_ipconfig_get_gateway_from_index(index);
 
 		if (gateway != NULL)
 			rtnl->newgateway(index, gateway);
@@ -578,9 +578,6 @@ static void process_newaddr(unsigned char family, unsigned char prefixlen,
 	void *src;
 	char ip_string[INET6_ADDRSTRLEN];
 
-	if (family != AF_INET && family != AF_INET6)
-		return;
-
 	if (family == AF_INET) {
 		struct in_addr ipv4_addr = { INADDR_ANY };
 
@@ -594,6 +591,8 @@ static void process_newaddr(unsigned char family, unsigned char prefixlen,
 			return;
 
 		src = &ipv6_address;
+	} else {
+		return;
 	}
 
 	if (inet_ntop(family, src, ip_string, INET6_ADDRSTRLEN) == NULL)
@@ -610,9 +609,6 @@ static void process_deladdr(unsigned char family, unsigned char prefixlen,
 	void *src;
 	char ip_string[INET6_ADDRSTRLEN];
 
-	if (family != AF_INET && family != AF_INET6)
-		return;
-
 	if (family == AF_INET) {
 		struct in_addr ipv4_addr = { INADDR_ANY };
 
@@ -626,6 +622,8 @@ static void process_deladdr(unsigned char family, unsigned char prefixlen,
 			return;
 
 		src = &ipv6_address;
+	} else {
+		return;
 	}
 
 	if (inet_ntop(family, src, ip_string, INET6_ADDRSTRLEN) == NULL)
@@ -1105,7 +1103,7 @@ static void rtnl_newnduseropt(struct nlmsghdr *hdr)
 	struct nd_opt_hdr *opt = (void *)&msg[1];
 	guint32 lifetime = -1;
 	const char **domains = NULL;
-	struct in6_addr *servers;
+	struct in6_addr *servers = NULL;
 	int nr_servers = 0;
 	int msglen = msg->nduseropt_opts_len;
 	char *interface;
