@@ -23,6 +23,8 @@
 #include <config.h>
 #endif
 
+#include <string.h>
+
 #include <gdbus.h>
 
 #include "connman.h"
@@ -44,9 +46,8 @@ enum connman_technology_state {
 	CONNMAN_TECHNOLOGY_STATE_UNKNOWN   = 0,
 	CONNMAN_TECHNOLOGY_STATE_OFFLINE   = 1,
 	CONNMAN_TECHNOLOGY_STATE_AVAILABLE = 2,
-	CONNMAN_TECHNOLOGY_STATE_BLOCKED   = 3,
-	CONNMAN_TECHNOLOGY_STATE_ENABLED   = 4,
-	CONNMAN_TECHNOLOGY_STATE_CONNECTED = 5,
+	CONNMAN_TECHNOLOGY_STATE_ENABLED   = 3,
+	CONNMAN_TECHNOLOGY_STATE_CONNECTED = 4,
 };
 
 struct connman_technology {
@@ -60,6 +61,10 @@ struct connman_technology {
 	gint blocked;
 	char *regdom;
 
+	connman_bool_t tethering;
+	char *tethering_ident;
+	char *tethering_passphrase;
+
 	struct connman_technology_driver *driver;
 	void *driver_data;
 };
@@ -133,79 +138,13 @@ void connman_technology_driver_unregister(struct connman_technology_driver *driv
 	driver_list = g_slist_remove(driver_list, driver);
 }
 
-void __connman_technology_add_interface(enum connman_service_type type,
-				int index, const char *name, const char *ident)
+static void tethering_changed(struct connman_technology *technology)
 {
-	GSList *list;
+	connman_bool_t tethering = technology->tethering;
 
-	switch (type) {
-	case CONNMAN_SERVICE_TYPE_UNKNOWN:
-	case CONNMAN_SERVICE_TYPE_SYSTEM:
-		return;
-	case CONNMAN_SERVICE_TYPE_ETHERNET:
-	case CONNMAN_SERVICE_TYPE_WIFI:
-	case CONNMAN_SERVICE_TYPE_WIMAX:
-	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
-	case CONNMAN_SERVICE_TYPE_CELLULAR:
-	case CONNMAN_SERVICE_TYPE_GPS:
-	case CONNMAN_SERVICE_TYPE_VPN:
-	case CONNMAN_SERVICE_TYPE_GADGET:
-		break;
-	}
-
-	connman_info("Create interface %s [ %s ]", name,
-				__connman_service_type2string(type));
-
-	for (list = technology_list; list; list = list->next) {
-		struct connman_technology *technology = list->data;
-
-		if (technology->type != type)
-			continue;
-
-		if (technology->driver == NULL)
-			continue;
-
-		if (technology->driver->add_interface)
-			technology->driver->add_interface(technology,
-							index, name, ident);
-	}
-}
-
-void __connman_technology_remove_interface(enum connman_service_type type,
-				int index, const char *name, const char *ident)
-{
-	GSList *list;
-
-	switch (type) {
-	case CONNMAN_SERVICE_TYPE_UNKNOWN:
-	case CONNMAN_SERVICE_TYPE_SYSTEM:
-		return;
-	case CONNMAN_SERVICE_TYPE_ETHERNET:
-	case CONNMAN_SERVICE_TYPE_WIFI:
-	case CONNMAN_SERVICE_TYPE_WIMAX:
-	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
-	case CONNMAN_SERVICE_TYPE_CELLULAR:
-	case CONNMAN_SERVICE_TYPE_GPS:
-	case CONNMAN_SERVICE_TYPE_VPN:
-	case CONNMAN_SERVICE_TYPE_GADGET:
-		break;
-	}
-
-	connman_info("Remove interface %s [ %s ]", name,
-				__connman_service_type2string(type));
-
-	for (list = technology_list; list; list = list->next) {
-		struct connman_technology *technology = list->data;
-
-		if (technology->type != type)
-			continue;
-
-		if (technology->driver == NULL)
-			continue;
-
-		if (technology->driver->remove_interface)
-			technology->driver->remove_interface(technology, index);
-	}
+	connman_dbus_property_changed_basic(technology->path,
+				CONNMAN_TECHNOLOGY_INTERFACE, "Tethering",
+						DBUS_TYPE_BOOLEAN, &tethering);
 }
 
 void connman_technology_tethering_notify(struct connman_technology *technology,
@@ -213,38 +152,37 @@ void connman_technology_tethering_notify(struct connman_technology *technology,
 {
 	DBG("technology %p enabled %u", technology, enabled);
 
+	if (technology->tethering == enabled)
+		return;
+
+	technology->tethering = enabled;
+
+	tethering_changed(technology);
+
 	if (enabled == TRUE)
 		__connman_tethering_set_enabled();
 	else
 		__connman_tethering_set_disabled();
 }
 
-static int set_tethering(const char *bridge, connman_bool_t enabled)
+static int set_tethering(struct connman_technology *technology,
+				const char *bridge, connman_bool_t enabled)
 {
-	GSList *list;
+	const char *ident, *passphrase;
 
-	for (list = technology_list; list; list = list->next) {
-		struct connman_technology *technology = list->data;
+	ident = technology->tethering_ident;
+	passphrase = technology->tethering_passphrase;
 
-		if (technology->driver == NULL)
-			continue;
+	if (technology->driver == NULL ||
+			technology->driver->set_tethering == NULL)
+		return -EOPNOTSUPP;
 
-		if (technology->driver->set_tethering)
-			technology->driver->set_tethering(technology,
+	if (technology->type == CONNMAN_SERVICE_TYPE_WIFI &&
+	    (ident == NULL || passphrase == NULL))
+		return -EINVAL;
+
+	return technology->driver->set_tethering(technology, ident, passphrase,
 							bridge, enabled);
-	}
-
-	return 0;
-}
-
-int __connman_technology_enable_tethering(const char *bridge)
-{
-	return set_tethering(bridge, TRUE);
-}
-
-int __connman_technology_disable_tethering(const char *bridge)
-{
-	return set_tethering(bridge, FALSE);
 }
 
 void connman_technology_regdom_notify(struct connman_technology *technology,
@@ -316,8 +254,6 @@ static const char *state2string(enum connman_technology_state state)
 		return "offline";
 	case CONNMAN_TECHNOLOGY_STATE_AVAILABLE:
 		return "available";
-	case CONNMAN_TECHNOLOGY_STATE_BLOCKED:
-		return "blocked";
 	case CONNMAN_TECHNOLOGY_STATE_ENABLED:
 		return "enabled";
 	case CONNMAN_TECHNOLOGY_STATE_CONNECTED:
@@ -395,13 +331,97 @@ static DBusMessage *get_properties(DBusConnection *conn,
 		connman_dbus_dict_append_basic(&dict, "Type",
 						DBUS_TYPE_STRING, &str);
 
+	connman_dbus_dict_append_basic(&dict, "Tethering",
+					DBUS_TYPE_BOOLEAN,
+					&technology->tethering);
+
+	if (technology->tethering_ident != NULL)
+		connman_dbus_dict_append_basic(&dict, "TetheringIdentifier",
+						DBUS_TYPE_STRING,
+						&technology->tethering_ident);
+
+	if (technology->tethering_passphrase != NULL)
+		connman_dbus_dict_append_basic(&dict, "TetheringPassphrase",
+						DBUS_TYPE_STRING,
+						&technology->tethering_passphrase);
+
 	connman_dbus_dict_close(&array, &dict);
 
 	return reply;
 }
 
+static DBusMessage *set_property(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	struct connman_technology *technology = data;
+	DBusMessageIter iter, value;
+	const char *name;
+	int type;
+
+	DBG("conn %p", conn);
+
+	if (dbus_message_iter_init(msg, &iter) == FALSE)
+		return __connman_error_invalid_arguments(msg);
+
+	dbus_message_iter_get_basic(&iter, &name);
+	dbus_message_iter_next(&iter);
+	dbus_message_iter_recurse(&iter, &value);
+
+	type = dbus_message_iter_get_arg_type(&value);
+
+	DBG("property %s", name);
+
+	if (g_str_equal(name, "Tethering") == TRUE) {
+		int err;
+		connman_bool_t tethering;
+		const char *bridge;
+
+		if (type != DBUS_TYPE_BOOLEAN)
+			return __connman_error_invalid_arguments(msg);
+
+		dbus_message_iter_get_basic(&value, &tethering);
+
+		if (technology->tethering == tethering)
+			return __connman_error_in_progress(msg);
+
+		bridge = __connman_tethering_get_bridge();
+		if (bridge == NULL)
+			return __connman_error_not_supported(msg);
+
+		err = set_tethering(technology, bridge, tethering);
+		if (err < 0)
+			return __connman_error_failed(msg, -err);
+
+	} else if (g_str_equal(name, "TetheringIdentifier") == TRUE) {
+		const char *str;
+
+		dbus_message_iter_get_basic(&value, &str);
+
+		if (technology->type != CONNMAN_SERVICE_TYPE_WIFI)
+			return __connman_error_not_supported(msg);
+
+		technology->tethering_ident = g_strdup(str);
+	} else if (g_str_equal(name, "TetheringPassphrase") == TRUE) {
+		const char *str;
+
+		dbus_message_iter_get_basic(&value, &str);
+
+		if (technology->type != CONNMAN_SERVICE_TYPE_WIFI)
+			return __connman_error_not_supported(msg);
+
+		if (strlen(str) < 8)
+			return __connman_error_invalid_arguments(msg);
+
+		technology->tethering_passphrase = g_strdup(str);
+	} else
+		return __connman_error_invalid_property(msg);
+
+	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+}
+
 static GDBusMethodTable technology_methods[] = {
-	{ "GetProperties", "", "a{sv}", get_properties },
+	{ "GetProperties", "",   "a{sv}", get_properties },
+	{ "SetProperty",   "sv", "",      set_property   },
 	{ },
 };
 
@@ -523,6 +543,73 @@ static void technology_put(struct connman_technology *technology)
 	g_free(technology);
 }
 
+void __connman_technology_add_interface(enum connman_service_type type,
+				int index, const char *name, const char *ident)
+{
+	struct connman_technology *technology;
+
+	switch (type) {
+	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+		return;
+	case CONNMAN_SERVICE_TYPE_ETHERNET:
+	case CONNMAN_SERVICE_TYPE_WIFI:
+	case CONNMAN_SERVICE_TYPE_WIMAX:
+	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
+	case CONNMAN_SERVICE_TYPE_CELLULAR:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
+	case CONNMAN_SERVICE_TYPE_GADGET:
+		break;
+	}
+
+	connman_info("Create interface %s [ %s ]", name,
+				__connman_service_type2string(type));
+
+	technology = technology_get(type);
+
+	if (technology == NULL || technology->driver == NULL
+			|| technology->driver->add_interface == NULL)
+		return;
+
+	technology->driver->add_interface(technology,
+					index, name, ident);
+}
+
+void __connman_technology_remove_interface(enum connman_service_type type,
+				int index, const char *name, const char *ident)
+{
+	struct connman_technology *technology;
+
+	switch (type) {
+	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+		return;
+	case CONNMAN_SERVICE_TYPE_ETHERNET:
+	case CONNMAN_SERVICE_TYPE_WIFI:
+	case CONNMAN_SERVICE_TYPE_WIMAX:
+	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
+	case CONNMAN_SERVICE_TYPE_CELLULAR:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
+	case CONNMAN_SERVICE_TYPE_GADGET:
+		break;
+	}
+
+	connman_info("Remove interface %s [ %s ]", name,
+				__connman_service_type2string(type));
+
+	technology = technology_find(type);
+
+	if (technology == NULL || technology->driver == NULL)
+		return;
+
+	if (technology->driver->remove_interface)
+		technology->driver->remove_interface(technology, index);
+
+	technology_put(technology);
+}
+
 static void unregister_technology(gpointer data)
 {
 	struct connman_technology *technology = data;
@@ -587,21 +674,17 @@ int __connman_technology_remove_device(struct connman_device *device)
 	return 0;
 }
 
-int __connman_technology_enable_device(struct connman_device *device)
+int __connman_technology_enable(enum connman_service_type type)
 {
 	struct connman_technology *technology;
-	enum connman_service_type type;
 
-	DBG("device %p", device);
-
-	technology = g_hash_table_lookup(device_table, device);
+	technology = technology_find(type);
 	if (technology == NULL)
 		return -ENXIO;
 
 	if (g_atomic_int_get(&technology->blocked))
 		return -ERFKILL;
 
-	type = __connman_device_get_service_type(device);
 	__connman_notifier_enable(type);
 
 	if (g_atomic_int_exchange_and_add(&technology->enabled, 1) == 0) {
@@ -612,22 +695,18 @@ int __connman_technology_enable_device(struct connman_device *device)
 	return 0;
 }
 
-int __connman_technology_disable_device(struct connman_device *device)
+int __connman_technology_disable(enum connman_service_type type)
 {
 	struct connman_technology *technology;
-	enum connman_service_type type;
 	GSList *list;
 
-	DBG("device %p", device);
-
-	type = __connman_device_get_service_type(device);
-	__connman_notifier_disable(type);
-
-	technology = g_hash_table_lookup(device_table, device);
+	technology = technology_find(type);
 	if (technology == NULL)
 		return -ENXIO;
 
 	if (g_atomic_int_dec_and_test(&technology->enabled) == TRUE) {
+		__connman_notifier_disable(type);
+
 		technology->state = CONNMAN_TECHNOLOGY_STATE_AVAILABLE;
 		state_changed(technology);
 	}
@@ -639,7 +718,7 @@ int __connman_technology_disable_device(struct connman_device *device)
 			return 0;
 	}
 
-	technology->state = CONNMAN_TECHNOLOGY_STATE_BLOCKED;
+	technology->state = CONNMAN_TECHNOLOGY_STATE_OFFLINE;
 	state_changed(technology);
 
 	return 0;
@@ -693,7 +772,7 @@ int __connman_technology_add_rfkill(unsigned int index,
 	if (g_atomic_int_exchange_and_add(&technology->blocked, 1) == 0) {
 		technology_blocked(technology, TRUE);
 
-		technology->state = CONNMAN_TECHNOLOGY_STATE_BLOCKED;
+		technology->state = CONNMAN_TECHNOLOGY_STATE_OFFLINE;
 		state_changed(technology);
 	}
 
@@ -736,7 +815,7 @@ int __connman_technology_update_rfkill(unsigned int index,
 			return 0;
 
 		technology_blocked(technology, blocked);
-		technology->state = CONNMAN_TECHNOLOGY_STATE_BLOCKED;
+		technology->state = CONNMAN_TECHNOLOGY_STATE_OFFLINE;
 		state_changed(technology);
 	} else {
 		if (g_atomic_int_dec_and_test(&technology->blocked) == FALSE)
